<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @author Tom Needham <tom@owncloud.com>
 * @author Frank Karlitscheck <frank@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Admin_Audit\Hooks;

use \OC\Files\Filesystem;
use \OCP\Util;
use \OCP\User;

class Handlers {
	static public function rename($params) {
		self::logOldNewPathAction('Rename', $params);
	}
	static public function create($params) {
		self::logSinglePathAction('Create', $params);
	}
	static public function copy($params) {
		self::logOldNewPathAction('Copy', $params);
	}
	static public function update($params) {
		self::logSinglePathAction('Update', $params);
	}
	static public function read($params) {
		self::logSinglePathAction('Read', $params);
	}
	static public function delete($params) {
		self::logSinglePathAction('Delete', $params);
	}

	// Trashbin 
	static public function trash_post_restore($params) {
		$params = array(
			Filesystem::signal_param_oldpath => $params['trashPath'],
			Filesystem::signal_param_newpath => $params['filePath'],
			);
		self::logOldNewPathAction('Restore', $params);
	}
	static public function trash_delete($params) {
		self::logSinglePathAction('DeleteFromTrash', $params);
	}

	// User actions
	static public function pre_login($params) {
		$user = $params['uid'];
		self::log('User '.$user.' attempted to log into ownCloud from IP address: '.\OC_Request::getRemoteAddress());
	}
	static public function post_login($params) {
		$user = $params['uid'];
		self::log('User '.$user.' logged into ownCloud from IP address: '.\OC_Request::getRemoteAddress());
	}
	static public function logout($params) {
		$user = User::getUser();
		self::log('User '.$user.' logged out of ownCloud');
	}
	static public function user_create($params) {
		self::log('User '.reset($params).' created by '.self::getUserOrIP());
	}
	static public function user_delete($params) {
		self::log('User '.reset($params).' deleted by '.self::getUserOrIP());
	}

	/**
	 * modifies the passed $params array:
	 * - replaces paths with their absolutePath
	 * - adds original owner of 'path' or 'oldpath'
	 * - adds user or IP accessing the file
	 * @param array $params
	 */
	static protected function resolveDetails(&$params) {
		if (isset($params[Filesystem::signal_param_path])) {
			$path = $params[Filesystem::signal_param_path];
			$params[Filesystem::signal_param_path] = Filesystem::getView()->getAbsolutePath($path);
			$params['owner'] = Filesystem::getOwner($path);
		} else {
			$oldpath = $params[Filesystem::signal_param_oldpath];
			$params[Filesystem::signal_param_oldpath] = Filesystem::getView()->getAbsolutePath($oldpath);
			$params['owner'] = Filesystem::getOwner($oldpath);
			$newpath = $params[Filesystem::signal_param_newpath];
			$params[Filesystem::signal_param_newpath] = Filesystem::getView()->getAbsolutePath($newpath);
		}
		$params['userOrIP'] = self::getUserOrIP();
	}
	
	/**
	 * returns the currently logged in user or the requests REMOTE_ADDR
	 * prepends result with 'user ' or 'IP '
	 * @return string
	 */
	static protected function getUserOrIP() {
		$user = User::getUser();
		if ($user) {
			$result = 'user ' . $user;
		} else {
			$result = 'IP ' . \OC_Request::getRemoteAddress();
		}
		return $result;
	}

	/**
	 * logs an action that takes two path parameters (rename, copy) in the form of:
	 * '$action "$oldpath" to "$newpath" by $user'.
	 * If original owner and user differ will append the original owner:
	 * '$action "$oldpath" to "$newpath" by $user, owner: $owner'.
	 * @param string $action
	 * @param array $params
	 */
	static protected function logOldNewPathAction($action, $params) {
		self::resolveDetails($params);
		$msg = $action.' "'.$params[Filesystem::signal_param_oldpath].'" to "'
						   .$params[Filesystem::signal_param_newpath].'" by '.$params['userOrIP'];
		if ($params['userOrIP'] !== $params['owner']) {
			$msg .= ', owner: '.$params['owner'];
		}
		self::log($msg);
	}

	/**
	 * logs an action that takes a single path parameter
	 * '$action "$oldpath" by $user'.
	 * If original owner and user differ will append the original owner:
	 * '$action "$oldpath" by $user, owner: $owner'.
	 * @param string $action
	 * @param array $params
	 */
	static protected function logSinglePathAction($action, $params) {
		self::resolveDetails($params);
		$msg = $action.' "'.$params[Filesystem::signal_param_path].'" by '.$params['userOrIP'];
		if ($params['userOrIP'] !== $params['owner']) {
			$msg .= ', owner: '.$params['owner'];
		}
		self::log($msg);
	}

	/**
	 * logs a message for 'admin_audit' at INFO level
	 * @param string $msg
	 */
	static protected function log($msg) {
		Util::writeLog('admin_audit', $msg, Util::INFO);
	}
}
