<?php
/**
 * ownCloud
 *
 * @author Owen Winkler
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Enterprise_Key;

use OC\HintException;
use OCP\Config;

class EnterpriseKey {
	/** @var string $org The org name of the key */
	protected $org;
	/** @var int $date A unix timestamp */
	protected $date;
	/** @var  string $rawCodes The raw code string */
	protected $rawCodes;
	/** @var string $codes The code string parsed into CRC chunks */
	protected $codes;
	/** @var string $hash The hash of the keycode */
	protected $hash;

	/**
	 * Class Constructor
	 * @param bool|string $licVal False to use the config, or a license key string to load
	 */
	public function __construct($licVal = false) {
		$licVal = $licVal ?: \OC_Config::$object->getValue('license-key', 'x-0-x-x');
		if($licVal === 'x-0-x-x') {
			// Got no license, create a temporary one.
			$expires = date('Ymd', time() + 2592000);
			$codes = hash("crc32b", date('Ymd', time() + 2592000)) . 'd642dfa0';
			$check = sprintf('%x', crc32("demozz{$codes}zz{$expires}"));
			$licVal = "demo-{$expires}-{$codes}-{$check}";
			Config::setSystemValue('license-key', $licVal);
		}
		list($this->org, $date, $this->rawCodes, $this->hash) = explode('-', $licVal);
		$this->codes = str_split(strtoupper($this->rawCodes), 8);
		$this->date = strtotime(substr($date, 0, 4) . '-' . substr($date, 4, 2) . '-' . substr($date, 6));
	}

	public function has($checkVal) {
		$hash = strtoupper(hash("crc32b", $checkVal));
		return in_array($hash, $this->codes);
	}

	/**
	 * Check the license key or look for a specific value in the bloom filter
	 * @param null|string $value Null or a value to search for in the bloom filter
	 * @return bool True if the requested value was found in the bloom filter
	 */
	public function check($value = null) {
		$check = sprintf('%x', crc32(strtolower($this->org) . 'zz' . strtolower($this->rawCodes) . 'zz' . date('Ymd', $this->date)));
		if($check != strtolower($this->hash)
			|| !$this->has(date('Ymd', $this->date))
		) {
			$l = \OC_L10N::get('enterprise_key');
			$err = $l->t('The enterprise license key is invalid.');
			\OC_Log::write('enterprise', $err, \OC_Log::FATAL);
			throw new HintException($err);
		}
		if($this->date < time() && $this->has('demo') ) {
			$l = \OC_L10N::get('enterprise_key');
			$err = $l->t('The enterprise license key has expired.');
			$hint = $l->t('Please contact <a href="mailto:sales@owncloud.com?subject=Renew+ownCloud+License">sales@owncloud.com</a> for a new license key.');
			\OC_Log::write('enterprise', $err, \OC_Log::FATAL);
			throw new HintException($err, $hint);
		}
		if(!empty($value)) {
			return $this->has($value);
		}
		return true;
	}

	public function expiresInDays() {
		return ceil(($this->date - time()) / 86400);
	}

	/**
	 * @param array $data The array of data supplied by the config.php hook \OCP\Config::js
	 */
	public static function getJs($data) {
		$key = new \OCA\Enterprise_Key\EnterpriseKey();
		$expiresIn = $key->expiresInDays();
		$data['array']['licenseKeyDaysLeft'] = $expiresIn;
		$l = \OC_L10N::get('enterprise_key');
		$message = '';
		if($key->has('demo') && $expiresIn > 0) {
			$message .= '<b>' . $l->t('Evaluation - expires in %d days.', array($expiresIn)) . '</b><br>';
		}
		elseif ($expiresIn < 45 && $expiresIn > 0) {
			$message .= $l->t('Your license key is about to expire (days remaining: %d). Please contact your administrator or <a href="mailto:sales@owncloud.com?subject=Renew+ownCloud+License">sales@owncloud.com</a>.', array($expiresIn));
		}
		elseif ($expiresIn < 0) {
			$message .= $l->t('This ownCloud license has expired. Please contact your administrator or <a href="mailto:sales@owncloud.com?subject=Renew+ownCloud+License">sales@owncloud.com</a> for a new license key.');
		}
		else {
			$message .= $l->t('Your license key will expire in %d days.', array($expiresIn));
		}
		if($message != '') {
			$data['array']['licenseKeyNotice'] = '"' . addslashes($message) . '"';
		}
	}
}
