<?php
/**
 * ownCloud
 *
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Files_Drop\Lib;

use OCP\Files\IRootFolder;
use OCP\Files\NotFoundException;
use OCP\IConfig;
use OCP\Files\Folder;
use OCP\IURLGenerator;
use OCP\Security\ISecureRandom;

/**
 * Class EndpointService
 *
 * @package OCA\Files_Drop\Lib
 */
class EndpointService {
	/** @var string */
	private $appName;
	/** @var IConfig */
	private $config;
	/** @var Folder */
	private $rootFolder;
	/** @var IURLGenerator */
	private $urlGenerator;
	/** @var ISecureRandom */
	private $secureRandom;
	/** @var HelperWrapper */
	private $helperWrapper;

	/**
	 * @param string $AppName
	 * @param IConfig $config
	 * @param IURLGenerator $urlGenerator
	 * @param IRootFolder $rootFolder
	 * @param ISecureRandom $secureRandom
	 * @param HelperWrapper $helperWrapper
	 */
	public function __construct($AppName,
								IConfig $config,
								IURLGenerator $urlGenerator,
								IRootFolder $rootFolder,
								ISecureRandom $secureRandom,
								HelperWrapper $helperWrapper) {
		$this->appName = $AppName;
		$this->config = $config;
		$this->urlGenerator = $urlGenerator;
		$this->rootFolder = $rootFolder;
		$this->secureRandom = $secureRandom;
		$this->helperWrapper = $helperWrapper;
	}

	/**
	 * @param string $user
	 * @throws \Exception If the user does not exist
	 * @return Endpoint
	 */
	public function getByUser($user) {
		return new Endpoint(
			$user,
			$this->appName,
			$this->config,
			$this->urlGenerator,
			$this->rootFolder->get($user . '/files/'),
			$this->helperWrapper
		);
	}

	/**
	 * @param string $token
	 * @throws NotFoundException
	 * @return Endpoint
	 */
	public function getByToken($token) {
		$users = $this->config->getUsersForUserValue($this->appName, 'token', $token);
		if(isset($users[0])) {
			return $this->getByUser($users[0]);
		}
		throw new NotFoundException();
	}

	/**
	 * Stores an endpoint
	 * @param Endpoint $endpoint
	 */
	public function save(Endpoint $endpoint) {
		$this->config->setUserValue($endpoint->getUser(), $this->appName, 'fileId', $endpoint->getFileId());
		$this->config->setUserValue($endpoint->getUser(), $this->appName, 'token', $endpoint->getToken());
	}

	/**
	 * Delete an endpoint
	 * @param Endpoint $endpoint
	 */
	public function delete(Endpoint $endpoint) {
		$this->config->deleteUserValue($endpoint->getUser(), $this->appName, 'fileId');
		$this->config->deleteUserValue($endpoint->getUser(), $this->appName, 'token');
	}

	/**
	 * @return string
	 */
	public function getUnusedToken() {
		$token = $this->secureRandom->getMediumStrengthGenerator()->generate(12,
			ISecureRandom::CHAR_LOWER
		);
		$conflicts = $this->config->getUsersForUserValue($this->appName, 'token', $token);
		while (!empty($conflicts)) {
			$token = $this->secureRandom->getMediumStrengthGenerator()->generate(12,
				ISecureRandom::CHAR_LOWER
			);
			$conflicts = $this->config->getUsersForUserValue($this->appName, 'token', $token);
		}

		return $token;
	}
}
