<?php
/**
 * ownCloud
 *
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Files_Drop\Tests\Unit\Controller;

use OCA\Files_Drop\Controller\PageController;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\DataResponse;
use OCP\AppFramework\Http\NotFoundResponse;
use OCP\AppFramework\Http\TemplateResponse;
use Test\TestCase;
use OCP\IRequest;
use OCA\Files_Drop\Lib\EndpointService;
use OCA\Files_Drop\Lib\HelperWrapper;
use OCP\IConfig;
use OCP\IURLGenerator;
use OC\Files\Node\Node;
use OCP\AppFramework\Http\JSONResponse;

/**
 * Class PageControllerTest
 *
 * @package OCA\Files_Drop\Tests\Unit\Controller
 */
class PageControllerTest extends TestCase {
	/** @var PageController */
	private $pageController;
	/** @var string */
	private $appName;
	/** @var IRequest */
	private $request;
	/** @var string */
	private $userId;
	/** @var EndpointService */
	private $endpointService;
	/** @var HelperWrapper */
	private $helperWrapper;
	/** @var IConfig */
	private $config;
	/** @var IURLGenerator */
	private $urlGenerator;
	/** @var Node */
	private $userFolder;

	public function setUp() {
		parent::setUp();
		$this->endpointService = $this->getMockBuilder('\OCA\Files_Drop\Lib\EndpointService')
			->disableOriginalConstructor()->getMock();
		$this->request = $this->getMockBuilder('\OCP\IRequest')
			->disableOriginalConstructor()->getMock();
		$this->helperWrapper = $this->getMockBuilder('\OCA\Files_Drop\Lib\HelperWrapper')
			->disableOriginalConstructor()->getMock();
		$this->appName = 'files_drop';
		$this->userId = 'björn.schießle';
		$this->config = $this->getMockBuilder('\OCP\IConfig')
			->disableOriginalConstructor()->getMock();
		$this->urlGenerator = $this->getMockBuilder('\OCP\IURLGenerator')
			->disableOriginalConstructor()->getMock();
		$this->userFolder = $this->getMockBuilder('\OCP\Files\Node')
			->disableOriginalConstructor()->getMock();
		$this->helperWrapper = $this->getMockBuilder('\OCA\Files_Drop\Lib\HelperWrapper')
			->disableOriginalConstructor()->getMock();

		$this->pageController = new PageController(
			$this->appName,
			$this->request,
			$this->userId,
			$this->endpointService,
			$this->helperWrapper
		);
	}

	public function testDropSuccessful() {
		$endpoint = $this->getMockBuilder('\OCA\Files_Drop\Lib\Endpoint')->setConstructorArgs([
			$this->userId,
			$this->appName,
			$this->config,
			$this->urlGenerator,
			$this->userFolder,
			$this->helperWrapper
		])->getMock();

		$this->endpointService
			->expects($this->once())
			->method('getByToken')
			->with('MyExistingToken')
			->will($this->returnValue($endpoint));
		$endpoint
			->expects($this->exactly(2))
			->method('getUser')
			->will($this->returnValue('User.123'));
		$endpoint
			->expects($this->once())
			->method('getToken')
			->will($this->returnValue('Token.123'));

		$expected = new TemplateResponse(
			$this->appName,
			'public',
			[
				'user' => 'User.123',
				'token' => 'Token.123',
				'maxUpload' => null,
			],
			'guest'
		);
		$this->assertEquals($expected, $this->pageController->drop('MyExistingToken'));
	}

	public function testDropException() {
		$this->endpointService
			->expects($this->once())
			->method('getByToken')
			->with('MyNotExistingToken')
			->will($this->throwException(new \Exception()));

		$this->assertEquals(new NotFoundResponse(), $this->pageController->drop('MyNotExistingToken'));
	}

	public function testUploadWithoutValidEndpoint() {
		$endpoint = $this->getMockBuilder('\OCA\Files_Drop\Lib\Endpoint')->setConstructorArgs([
			$this->userId,
			$this->appName,
			$this->config,
			$this->urlGenerator,
			$this->userFolder,
			$this->helperWrapper
		])->getMock();

		$this->endpointService
			->expects($this->once())
			->method('getByToken')
			->with('FancyToken')
			->will($this->returnValue($endpoint));
		$endpoint
			->expects($this->once())
			->method('getUser')
			->will($this->returnValue(''));

		$expected = new DataResponse(['message' => 'Endpoint not found'], Http::STATUS_NOT_FOUND);
		$this->assertEquals($expected, $this->pageController->upload('FancyToken'));
	}

	public function testUploadNoFile() {
		$endpoint = $this->getMockBuilder('\OCA\Files_Drop\Lib\Endpoint')->setConstructorArgs([
			$this->userId,
			$this->appName,
			$this->config,
			$this->urlGenerator,
			$this->userFolder,
			$this->helperWrapper
		])->getMock();

		$this->endpointService
			->expects($this->once())
			->method('getByToken')
			->with('FancyToken')
			->will($this->returnValue($endpoint));
		$endpoint
			->expects($this->once())
			->method('getUser')
			->will($this->returnValue($this->userId));

		$expected = new DataResponse(['message' => 'No file uploaded'], Http::STATUS_UNPROCESSABLE_ENTITY);
		$this->assertEquals($expected, $this->pageController->upload('FancyToken'));
	}

	public function testUploadFile() {
		$endpoint = $this->getMockBuilder('\OCA\Files_Drop\Lib\Endpoint')->setConstructorArgs([
			$this->userId,
			$this->appName,
			$this->config,
			$this->urlGenerator,
			$this->userFolder,
			$this->helperWrapper
		])->getMock();
		$folderView = $this->getMockBuilder('\OCP\Files\Folder')->disableOriginalConstructor()->getMock();
		$file = $this->getMockBuilder('\OCP\Files\File')->disableOriginalConstructor()->getMock();

		$this->endpointService
			->expects($this->once())
			->method('getByToken')
			->with('FancyToken')
			->will($this->returnValue($endpoint));
		$endpoint
			->expects($this->once())
			->method('getUser')
			->will($this->returnValue($this->userId));
		$endpoint
			->expects($this->once())
			->method('getFolderView')
			->will($this->returnValue($folderView));
		$this->request
			->expects($this->once())
			->method('getUploadedFile')
			->with('file')
			->will($this->returnValue(['name' => 'MyFileName', 'tmp_name' => __FILE__]));
		$folderView
			->expects($this->once())
			->method('getNonExistingName')
			->with('MyFileName')
			->will($this->returnValue('MyFileName (1)'));
		$folderView
			->expects($this->once())
			->method('newFile')
			->with('MyFileName (1)')
			->will($this->returnValue($file));
		$file
			->expects($this->once())
			->method('putContent');

		$expected = new JSONResponse(['file' => 'MyFileName (1)']);
		$expected->setHeaders(['Content-Type' => 'text/plain']);
		$this->assertEquals($expected, $this->pageController->upload('FancyToken'));
	}

	public function testUploadFileServerError() {
		$endpoint = $this->getMockBuilder('\OCA\Files_Drop\Lib\Endpoint')->setConstructorArgs([
			$this->userId,
			$this->appName,
			$this->config,
			$this->urlGenerator,
			$this->userFolder,
			$this->helperWrapper
		])->getMock();
		$folderView = $this->getMockBuilder('\OCP\Files\Folder')->disableOriginalConstructor()->getMock();
		$file = $this->getMockBuilder('\OCP\Files\File')->disableOriginalConstructor()->getMock();

		$this->endpointService
			->expects($this->once())
			->method('getByToken')
			->with('FancyToken')
			->will($this->returnValue($endpoint));
		$endpoint
			->expects($this->once())
			->method('getUser')
			->will($this->returnValue($this->userId));
		$endpoint
			->expects($this->once())
			->method('getFolderView')
			->will($this->returnValue($folderView));
		$this->request
			->expects($this->once())
			->method('getUploadedFile')
			->with('file')
			->will($this->returnValue(['name' => 'MyFileName', 'tmp_name' => __FILE__]));
		$folderView
			->expects($this->once())
			->method('getNonExistingName')
			->with('MyFileName')
			->will($this->returnValue('MyFileName (1)'));
		$folderView
			->expects($this->once())
			->method('newFile')
			->with('MyFileName (1)')
			->will($this->returnValue($file));
		$file
			->expects($this->once())
			->method('putContent')
			->will($this->throwException(new \Exception()));

		$expected = new DataResponse(['message' => 'Server error'], Http::STATUS_INTERNAL_SERVER_ERROR);
		$this->assertEquals($expected, $this->pageController->upload('FancyToken'));
	}

	public function testUploadFolder() {
		$endpoint = $this->getMockBuilder('\OCA\Files_Drop\Lib\Endpoint')->setConstructorArgs([
			$this->userId,
			$this->appName,
			$this->config,
			$this->urlGenerator,
			$this->userFolder,
			$this->helperWrapper
		])->getMock();
		$folderView = $this->getMockBuilder('\OCP\Files\Folder')->disableOriginalConstructor()->getMock();
		$file = $this->getMockBuilder('\OCP\Files\File')->disableOriginalConstructor()->getMock();

		$this->endpointService
			->expects($this->once())
			->method('getByToken')
			->with('FancyToken')
			->will($this->returnValue($endpoint));
		$endpoint
			->expects($this->once())
			->method('getUser')
			->will($this->returnValue($this->userId));
		$endpoint
			->expects($this->once())
			->method('getFolderView')
			->will($this->returnValue($folderView));
		$this->request
			->expects($this->once())
			->method('getUploadedFile')
			->with('file')
			->will($this->returnValue(['name' => 'MyFileName', 'tmp_name' => __FILE__]));
		$folderView
			->expects($this->once())
			->method('getNonExistingName')
			->with('MyFileName')
			->will($this->returnValue('MyFileName (1)'));
		$folderView
			->expects($this->once())
			->method('newFile')
			->with('MyFileName (1)')
			->will($this->returnValue($file));
		$file
			->expects($this->once())
			->method('putContent')
			->will($this->throwException(new \Exception()));

		$expected = new DataResponse(['message' => 'Server error'], Http::STATUS_INTERNAL_SERVER_ERROR);
		$this->assertEquals($expected, $this->pageController->upload('FancyToken', 'MyFolder'));
	}

}
