<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\ObjectStore;
use AmazonS3;
use OCP\Files\ObjectStore\IObjectStore;

require_once __DIR__ . '/../3rdparty/autoload.php';

class S3 implements IObjectStore {

	/**
	 * @var \AmazonS3
	 */
	private $connection;

	/**
	 * @var array
	 */
	private $params;

	/**
	 * @var string
	 */
	private $bucket;

	/**
	 * @var int
	 */
	private $timeout = 15;

	public function __construct($params) {
		if (!isset($params['key']) || !isset($params['secret']) ) {
			throw new \Exception("Access Key and Secret have to be configured.");
		}

		$this->bucket = $params['bucket'];

		if (!isset($params['use_ssl'])) {
			$params['use_ssl'] = true;
		}
		if (!isset($params['ssl_verification'])) {
			$params['ssl_verification'] = true;
		}
		//$scheme = ($params['use_ssl'] === 'false') ? 'http' : 'https';
		$this->timeout = ( ! isset($params['timeout'])) ? 15 : $params['timeout'];
		if (!isset($params['port']) || $params['port'] === '') {
			$params['port'] = ($params['use_ssl'] === 'false') ? 80 : 443;
		}
		//$base_url = $scheme.'://'.$params['hostname'].':'.$params['port'].'/';
		$this->params = $params;
	}

	protected function init() {
		if ($this->connection) {
			return;
		}
		$options = array(
			'key' => $this->params['key'],
			'secret' => $this->params['secret'],
			'region' => $this->params['region']
		);
		if (isset($this->params['base_url'])) {
			$options['base_url'] = $this->params['base_url'];
		}
		$this->connection = new AmazonS3($options);

		// ceph config
		if (isset($this->params['hostname'])) {
			$this->connection->set_hostname($this->params['hostname']);
			$this->connection->allow_hostname_override(false);
		}

		$this->connection->use_ssl = $this->params['use_ssl'];
		$this->connection->ssl_verification = $this->params['ssl_verification'];
		//FIXME $this->connection->enable_path_style();

		if (!$this->connection->if_bucket_exists($this->bucket)) {
			try {
				$result = $this->connection->create_bucket($this->bucket, $this->params['region']);
			} catch (\Exception $e) {
				\OCP\Util::writeLog('files_external', $e->getMessage(), \OCP\Util::ERROR);
				throw new \Exception("Creation of bucket failed.");
			}
		}
	}

	/**
	 * @return string the container name where objects are stored
	 */
	public function getStorageId() {
		return $this->bucket;
	}


	/**
	 * @param string $urn the unified resource name used to identify the object
	 * @param resource $stream stream with the data to write
	 * @throws Exception when something goes wrong
	 */
	public function writeObject($urn, $stream) {
		$this->init();
		$opt = array();
		if ($stream) {
			$opt['fileUpload'] = $stream;
		} else {
			$opt['headers'] = array(
				'ContentLength' => 0
			);
		}
		$result = $this->connection->create_object(
			$this->bucket,
			$urn,
			$opt
		);
		if ($stream) {
			fclose($stream);
		}
		$this->checkResult($result);
	}
	public function deleteObject($urn) {
		$this->init();
		$result = $this->connection->delete_object(
			$this->bucket,
			$urn
		);
		$this->checkResult($result);
	}


	/**
	 * @param string $urn the unified resource name used to identify the object
	 * @return resource stream with the read data
	 * @throws Exception from when something goes wrong
	 */
	public function readObject($urn) {
		$this->init();

		$tmpStream = fopen('php://temp', 'r+');

		$result = $this->connection->get_object(
			$this->bucket,
			$urn,
			array('fileDownload' => $tmpStream)
		);
		$this->checkResult($result);
		rewind($tmpStream);
		return $tmpStream;
	}

	private function checkResult(\CFResponse $result) {
		if (!$result->isOK()) {
			$message = json_encode($result->body);
			if (!$message && $result->status === 404) {
				$message = 'Not Found';
			}
			throw new \Exception($message, $result->status);
		}
	}
}
