<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

/**
 * REQUIREMENTS:
 * - libsmbclient and libsmbclient-dev packages installed
 * - libsmbclient-php bindings installed and libsmbclient.so active for php
 * - HOME variable accessible for php (due to a bug in libsmbclient-php)
 */

namespace OCA\windows_network_drive\lib;
use \Icewind\SMB\Exception\AlreadyExistsException;
use \Icewind\SMB\Exception\NotFoundException;
use \Icewind\SMB\NativeServer;
use \OC\Files\Stream\Dir;
/**
 * FS class to connect to windows network drives
 */
class WND extends \OC\Files\Storage\Common {
    /** Enable debug */
    private static $enableLogs = FALSE;
    /** Password for the connection */
    private $password;
    /** User to connect */
    private $user;
    /** Host where the connection should be made */
    private $host;
    /** The folder that will act as root */
    private $root;
    /** The share name that we'll use */
    private $shareName;

    /** is sharing active for this FS? */
    private $isSharingActive = false;
    /** a cache for the dir call */
    private $dirCache = array();
    private static $tempFiles = array();
    /** a cache for attributes */
    private $xattrCache = array();

    /** Reference to a SMB server (check 3rdparty native server) */
    private $server;
    /** Reference to a SMB share (check 3rdparty native share) */
    private $share;


    /**
     * Construct the object. The required params are: "host", "user", "password" and "share".
     * "root" is optional, defaulting to '/'
     *
     * @param array $params an array with the required params
     */
    public function __construct($params) {
        // if user is null we convert username and password to '' to try anonymous login
        $params = $this->fixConstructParams($params);
        $this->isSharingActive = $params['oc_sharing'];
        if (isset($params['host']) && isset($params['share'])) {
            $params['share'] = trim($params['share'], '/');
            $this->shareName = $params['share'];
            $this->host = $params['host'];
            $this->user = $params['user'];
            $this->password = $params['password'];
            $this->root = $params['root'];
            $this->server = new NativeServer($this->host, $this->user, $this->password);
            $this->share = $this->server->getShare($params['share']);
        } else {
            throw new \Exception('invalid parameters in the wnd constructor');
        }
    }

    private function fixConstructParams($params) {
        // if user is null we convert username and password to '' to try anonymous login
        $params['user'] = Utils::getDefault($params, 'user', '');
        $params['password'] = Utils::getDefault($params, 'password', '');

        //make sure the root end like "/something/" or "/something/more/"
        $params['root'] = Utils::getDefault($params, 'root', '');
        $params['root'] = '/' . trim($params['root'], '/') . '/';
        //fix special case
        if ($params['root'] === '//') {
            $params['root'] = '/';
        }

        $params['oc_sharing'] = (bool) Utils::getDefault($params, 'oc_sharing', false);
        return $params;
    }

    /**
     * Get the username trying to access to the windows drive. It might be different than the
     * current ownCloud user
     *
     * @return string the username
     */
    public function getUser() {
        return $this->user;
    }

    /**
     * Get the host we're trying to access
     *
     * @return string the host
     */
    public function getHost() {
        return $this->host;
    }

    /**
     * Get the sharename where we're trying to access
     *
     * @return string the share name
     */
    public function getShareName() {
        return $this->shareName;
    }

    /**
     * Get the folder that will act as our root
     *
     * @return string the root folder
     */
    public function getRoot() {
        return $this->root;
    }

    /**
     * Get the password we'll to to connect to the windows drive. Use it only for unittest
     *
     * @return string the password we'll use
     */
    public function getPassword() {
        return $this->password;
    }

    /**
     * Get the FS id (for ownCloud purposes)
     *
     * @return string the FS id
     */
    public function getId(){
        return 'wnd::' . $this->getUser() . '@' . $this->getHost() . '/' . $this->getShareName() . '/' . $this->getRoot();
    }

    private function dir($path) {
        $path = trim($path, '/');
        try {
            if (!isset($this->dirCache[$path])) {
                $this->dirCache[$path] = array();
                $dirData = $this->share->dir($path);
                foreach ($dirData as $fileInfo) {
                    $this->dirCache[$path][$fileInfo->getName()] = $fileInfo;
                }
            }
            return $this->dirCache[$path];
        } catch (NotFoundException $e) {
            return array();
        }
    }

    private function removeFromCache($path) {
        $path = trim($path, '/');
        if (isset($this->dirCache[$path])) {
            unset($this->dirCache[$path]);
        }

        if ($this->isRootDir(dirname($path))) {
            // if we need to remove the root, wipe the cache
            $this->dirCache = array();
        } else {
            if (isset($this->dirCache[dirname($path)])) {
                unset($this->dirCache[dirname($path)]);
            }
        }
    }


    /**
     * Create a directory. Notice that all the parent directories should exists
     *
     * @param string $path the directory we want to create
     * @return bool true if the directory is created, false otherwise
     */
    public function mkdir($path) {
        $path = trim($path, '/');
        try {
            if (!$this->file_exists($path)) {
                $this->removeFromCache($this->root . $path);
                $this->share->mkdir($this->root . $path);
                return true;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            return false;
        }
    }

    /**
     * Remove the directory and its contents
     *
     * @param string $path the FS path that we want to delete
     * @return bool true if deleted, false otherwise
     */
    public function rmdir($path) {
        $path = trim($path, '/');
        try {
            $content = $this->dir($this->root . $path);
            foreach ($content as $name => $fileInfo) {
                if (!$fileInfo->isDirectory()) {
                    $this->unlink($path . '/' . $name);
                } else {
                    $this->rmdir($path . '/' . $name);
                }
            }
            $this->share->rmdir($this->root . $path);
            $this->removeFromCache($this->root . $path);
            return true;
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            return false;
        }
    }

    /**
     * Open the directory
     *
     * @param string $path FS path
     * @return resource a directory resource
     */
    public function opendir($path) {
        $path = trim($path, '/');
        try {
            $files = $this->dir($this->root . $path);
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            $files = array();
        }
        Dir::register('wnd:' . $path, array_keys($files));
        return opendir('fakedir://wnd:' . $path);
    }

    /**
     * Get the file type of the path
     *
     * @param string $path FS path
     * @return string the type of the path (usually 'dir' or 'file')
     */
    public function filetype($path) {
        $path = trim($path, '/');
        if ($this->isRootDir($path)) {
            return 'dir';
        }

        $stat = $this->stat($path);
        if ($stat) {
            return $stat['type'];
        } else {
            return false;
        }
    }

    /**
     * Check the the file defined by path exists
     *
     * @param string $path FS path
     * @return bool true if the file exists, false otherwise
     */
    public function file_exists($path) {
        $path = trim($path, '/');
        if ($this->isRootDir($path) && $this->remoteIsShare()) {
            // refering to a share (not folder) -> return true
            return true;
        }

        try {
            $parent = dirname($this->root . $path);
            $files = $this->dir($parent);
            return isset($files[basename($this->root . $path)]);
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            return false;
        }
    }

    /**
     * Remove the path. If the path is a directory, this function falls back to the rmdir
     * function
     *
     * @param string $path FS path to be deleted
     * @return bool true if the path no longer exists, false otherwise
     */
    public function unlink($path) {
        $path = trim($path, '/');
        try {
            if ($this->is_dir($path)) {
                return $this->rmdir($path);
            } else {
                $this->removeFromCache($this->root . $path);
                return $this->share->del($this->root . $path);
            }
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            return false;
        }
    }

    /**
     * Open a file resouce
     *
     * @param string $path file path to be opened
     * @param string $mode open the file in this mode
     * @return resource an opened resource to read or write
     */
    public function fopen($path, $mode) {
        $path = trim($path, '/');
        $tmpFile = \OCP\Files::tmpFile();
        try {
            $this->share->get($this->root . $path, $tmpFile);
        } catch (\Exception $e) {
            if ($mode === 'r' or $mode === 'rb' or $mode === 'rw' or $mode === 'rwb') {
                \OCP\Util::logException('wnd', $e);
                return false;
            }
        }
        if ($mode === 'r' or $mode === 'rb') {
            return fopen($tmpFile, $mode);
        } else {
            self::$tempFiles[$tmpFile] = $path;
            \OC\Files\Stream\Close::registerCallback($tmpFile, array($this, 'writeBack'));
            return fopen('close://' . $tmpFile, $mode);
        }
    }

    /**
     * Writes the file.
     *
     * NOTICE: This function is for a callback in the fopen function, so it's not intended to use
     * outside
     */
    public function writeBack($tmpFile) {
        if (isset(self::$tempFiles[$tmpFile])) {
            try {
                $this->uploadFile($tmpFile, self::$tempFiles[$tmpFile]);
            } catch (\Exception $e) {
                \OCP\Util::logException('wnd', $e);
                unlink($tmpFile);

                $filename = basename(self::$tempFiles[$tmpFile]);
                $l = \OC_L10N::get('windows_network_drive');
                $errorMessage = $l->t('Couldn\'t upload file: %s', $filename);
                throw new \OCP\Files\NotPermittedException($errorMessage, $e->getCode(), $e);
            }
            unlink($tmpFile);
        }
    }

    /**
     * Write the data in the specified path (file)
     *
     * @param string $path the target file
     * @param string $data the data to be written
     */
    public function file_put_contents($path, $data) {
        $path = trim($path, '/');
        $tmpFile = \OCP\Files::tmpFile();
        file_put_contents($tmpFile, $data);
        try {
            $this->share->put($tmpFile, $this->root . $path);
            $this->removeFromCache($this->root . $path);
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
        }
        unlink($tmpFile);
    }


    /**
     * "Touch" the path. This function will only create a empty file if not exists or update
     * the mtime to the latest if it exists. Setting the mtime is not supported
     *
     * @param string $path path to be "touched"
     * @param mixed $mtime null to be updated to the latest, if it's not null, it will be ignored
     * @return bool true if file exists and mtime updated
     */
    public function touch($path, $time = null) {
        if (!is_null($time)) {
            return false;
        }
        if (!$this->file_exists($path)) {
            $this->file_put_contents($path, '');
            // removeFromCache is called in the file_put_contents
            return $this->file_exists($path);
        } else {
            return false;
        }
    }

    /**
     * Copy the contents from one file to another
     *
     * @param string $source the source file
     * @param string $target the target file
     */
    public function copy($source, $target) {
        $tmpFile = \OCP\Files::tmpFile();
        try {
            $this->share->get($this->root . $source, $tmpFile);
            $this->share->put($tmpFile, $this->root . $target);
            $this->removeFromCache($this->root . $target);
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
        }
        unlink($tmpFile);
    }

    /**
     * Get the mimetype of the path or false if the path not exists
     *
     * @param string $path the target path
     * @return string|false the mimetype or false
     */
    public function getMimeType($path) {
        if ($this->is_dir($path)) {
            return 'httpd/unix-directory';
        }
        if (!$this->file_exists($path)) {
            return false;
        }
        return \OC_Helper::getFileNameMimeType($path);
    }

    /**
     * Upload a file to the external storage
     *
     * @param string $path the local file that should be uploaded
     * @param string $target the external path where the file will be placed
     * @return bool true if the file is copied correctly, false otherwise
     */
    public function uploadFile($path, $target) {
        $this->removeFromCache($this->root . $target);
        $this->share->put($path, $this->root . $target);
    }

    /**
     * Rename the files
     *
     * @param string $path1 the old name of the path
     * @param string $path2 the new name of the path
     * @return bool true if the rename is successful, false otherwise
     */
    public function rename($source, $target) {
        try {
            $result = $this->share->rename($this->root . $source, $this->root . $target);
            $this->removeFromCache($this->root . $source);
            $this->removeFromCache($this->root . $target);
        } catch (AlreadyExistsException $e) {
            $this->unlink($target);
            $result = $this->share->rename($this->root . $source, $this->root . $target);
            $this->removeFromCache($this->root . $source);
            $this->removeFromCache($this->root . $target);
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            $result = false;
        }
        return $result;
    }

    /**
     * perform the stat call over the file
     *
     * @param string $path the FS path
     * @param array|false an array with information or false in case of error
     */
    public function stat($path) {
        $path = trim($path, '/');
        try {
            if ($this->remoteIsShare() && $this->isRootDir($path)) { //mtime doesn't work for shares
                return array('mtime' => $this->shareMTime(), 'size' => 0, 'type' => 'dir');
            } else {
                $parent = dirname($this->root . $path);
                $files = $this->dir($parent);
                if (isset($files[basename($this->root . $path)])) {
                    $file = $files[basename($this->root . $path)];
                    return array('mtime' => $file->getMTime(),
                                'size' => $file->getSize(),
                                'type' => ($file->isDirectory()) ? 'dir' : 'file');
                } else {
                    return false;
                }
            }
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            return false;
        }
    }

    /**
     * Check the path has updated in the given time. The check is usually based on the mtime
     * of the path, so if the path's filemtime is greater than $time, the file has updated
     *
     * @param string $path the FS path
     * @param int $time timestamp to compare against
     * @throw \OCP\Files\StorageNotAvailableException
     * @return bool true if the file has updated, false otherwise
     */
    public function hasUpdated($path,$time) {
        $stat = $this->stat($path);
        if ($stat === false) {
            \OC_Log::write('wnd', 'hasUpdated failed -> storage not available', \OC_Log::ERROR);
            throw new \OCP\Files\StorageNotAvailableException();
            // if no stat consider a the file has updated to force a rescan
            //return true;
        } else {
            $currentMtime = $stat['mtime'];
            return $currentMtime > $time;
        }
    }

    /**
     * get the best guess for the modification time of the share
     *
     * @param string $path the target path
     * @return array the first element of the array is the calculated mtime for the folder, and
     * the second is the list of readable files (the list contains the filename as key and the
     * mtime for the file as value)
     */
    private function shareMTime() {
        $files = $this->dir($this->root);
        $lastMtime = 0;
        foreach ($files as $file) {
            if ($file->getMTime() > $lastMtime) {
                $lastMtime = $file->getMTime();
            }
        }
        return $lastMtime;
    }

    /**
     * Check if the path is our root dir (not the smb one)
     *
     * @param string $path the path
     * @return bool true if it's root, false if not
     */
    private function isRootDir($path) {
        return ($path == '' || $path == '/' || $path == '.');
    }

    /**
     * Check if our root points to a smb share
     *
     * @return bool true if our root points to a share false otherwise
     */
    private function remoteIsShare() {
        return $this->getShareName() && (!$this->getRoot() || $this->getRoot() === '/');
    }

    /**
     * Check if our root points to a smb path
     *
     * @return bool true if our root points to a path false otherwise
     */
    private function remoteIsFolder() {
        return $this->getShareName() && ($this->getRoot() && $this->getRoot() !== '/');
    }

    /**
     * Check where our root points to remotely. It can be a smb share ("share") or one of the
     * inner folder ("folder")
     *
     * @return string 'share' or 'folder'
     */
    public function remoteType() {
        if ($this->remoteIsShare()) return 'share';
        if ($this->remoteIsFolder()) return 'folder';
    }

    /**
     * Check if the path is readable
     *
     * @param string $path the path
     * @return bool true if it's readable, false if not
     */
    public function isReadable($path) {
        $fileInfoOperation = (($info = $this->getFileInfoOperation($path, 'isReadOnly')) === null) ? false : !$info;
        self::log('isCreatable: '.$path.' - '.$fileInfoOperation, \OC_Log::DEBUG);
        if ($this->isRootDir($path)) {
            return true;
        }
        return $fileInfoOperation;
    }

    /**
     * Check if the path is updatable
     *
     * @param string $path the path
     * @return bool true if it's updatable, false if not
     */
    public function isUpdatable($path) {
        $fileInfoOperation = (($info = $this->getFileInfoOperation($path, 'isReadOnly')) === null) ? false : !$info;
        self::log('isCreatable: '.$path.' - '.$fileInfoOperation, \OC_Log::DEBUG);
        return $fileInfoOperation;
    }

    /**
     * Check if the path is creatable
     *
     * @param string $path the path
     * @return bool true if it's creatable, false if not
     */
    public function isCreatable($path) {
        $fileInfoOperation = (($info = $this->getFileInfoOperation($path, 'isReadOnly')) === null) ? false : !$info;
        self::log('isCreatable: '.$path.' - '.$fileInfoOperation, \OC_Log::DEBUG);
        if (!$this->isRootDir($path)) {
            // if the path isn't root, use the default behaviour
            return parent::isCreatable($path);
        }
        // if path is root check that the folder isn't read-only
        return $fileInfoOperation;
    }

    private function getFileInfoOperation($path, $operation) {
        if (isset($this->xattrCache[$path])) {
             $nativeFI = $this->xattrCache[$path];
        } else {
             $realPath = $this->root . $path;
             $nativeFI = new \Icewind\SMB\NativeFileInfo($this->share, $realPath, basename($realPath));
             $this->xattrCache[$path] = $nativeFI;
        }

        try {
            return call_user_func(array($nativeFI, $operation));
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            return null;
        }
    }

    /**
     * Check if the path is sharable. Currently sharing is disabled so it will always
     * return false
     *
     * @param string $path the path
     * @return false
     */
    public function isSharable($path) {
        return $this->isSharingActive;
    }

    /**
     * @param string $message
     */
    public static function log($message, $level, $from='wnd') {
        if(self::$enableLogs === true){
            \OC_Log::write($from, $message, $level);
        }
    }

    /**
     * A simple function to test connectivity. This functions might throw exceptions
     */
    public function test() {
        $this->share->dir(trim($this->root, '/'));
        return true;
    }
}
